grammar FilterExpression;

options {
output=AST;
ASTLabelType=CommonTree;
}

tokens{
 DATE;
 TIME;
}

@header { 
package org.ait.table.filter; 
}

@lexer::header {
  package org.ait.table.filter;
}

@members{
private boolean numberColumn = false;
private boolean stringColumn = false;
private boolean dateColumn = true;

public void setStringColumn(boolean b){
	stringColumn = b;
}

public void setNumberColumn(boolean b){
	numberColumn = b;
}

public void setDateColumn(boolean b){
	dateColumn = b;
}

///**@Override
  protected Object recoverFromMismatchedToken(IntStream input, int ttype, BitSet follow) throws RecognitionException {
    throw new MismatchedTokenException(ttype, input);
  }

  @Override
  public Object recoverFromMismatchedSet(IntStream input, RecognitionException e, BitSet follow) throws RecognitionException {
    throw e;
  }
  //*/
}

@lexer::members{
private boolean numberColumn = false;

public void setNumberColumn(boolean b){
	numberColumn = b;
}
}

//**
@rulecatch {
    catch (RecognitionException e) {
        throw e;
    }
}
//*/

formula	:	disjunction
	;	

disjunction :	conjuction (OR ^ conjuction)*
	;
	
conjuction :	predicate(AND ^ predicate)*
	;

predicate :	'is.na' ^
	|	'is.null'^
	|	{numberColumn}?=> numericPredicate
	|	{dateColumn}?=> datePredicate
	|	textPredicate
	|	NOT ^ predicate
	|	LP ! disjunction RP !
	;
	
textPredicate : 'contains' ^ STRING_LITERAL
	| 'equals' ^ TO? STRING_LITERAL
	;

numericPredicate:('='|'<'|'<='|'>'|'>=') ^ number
	|	number TO ^ number
	;

number: 	INTEGER
	|	SIGNEDINTEGER
	| 	FLOAT
	;

datePredicate
	:	('='|'<'|'<='|'>'|'>=') ^ date
	|	date TO ^ date
	|	('='|'<'|'<='|'>'|'>=') ^ time
	|	time TO ^ time
	;

time	:	date	(hour=INTEGER | hour=DATENUM) ':' (minute=INTEGER | minute=DATENUM) ':' (second=INTEGER | second=DATENUM) -> ^(TIME date $hour $minute $second);

date	:	(year=INTEGER | year=DATENUM) '-' (month=INTEGER | month=DATENUM) '-' (day=INTEGER | day=DATENUM)-> ^(DATE $year $month $day)
	|	(year=INTEGER | year=DATENUM) '.' (month=INTEGER | month=DATENUM) '.' (day=INTEGER | day=DATENUM)-> ^(DATE $year $month $day)
	;	


DATENUM: '0' '1'..'9';

INTEGER: '0'| '1'..'9' DIGIT*;

SIGNEDINTEGER
	:	{numberColumn}?=>SIGN INTEGER;

FLOAT	:	{numberColumn}?=>(INTEGER|SIGNEDINTEGER) '.' DIGIT+;

fragment SIGN	:	'+' | '-';


STRING_LITERAL: '"' NONCONTROL_CHAR* '"';

fragment NONCONTROL_CHAR: LETTER | DIGIT | SYMBOL | SPACE;
fragment LETTER: LOWER | UPPER;
fragment LOWER: 'a'..'z';
fragment UPPER: 'A'..'Z';
fragment DIGIT: '0'..'9';
fragment SPACE: ' ' | '\t';
// Note that SYMBOL does not include the double-quote character.
fragment SYMBOL: '!' | '#'..'/' | ':'..'@' | '['..'`' | '{'..'~';

AND	:	'and' ;
OR	:	'or' ;
NOT	:	'not'  ;
TO	:	'to' ;
LP	:	'(' ;
RP	:	')' ;

WS  :   ( ' '
        | '\t'
        | '\r'
        | '\n'
        ) {$channel=HIDDEN;}
    ;